@@ -20,6 +20,22 @@
 #'   \code{\link{readXenium}} for options and multiple images can be specified.
 #'   If \code{NULL}, then the default from the read function for the technology
 #'   will be used.
+#' @param sparse Logical, whether the gene count matrix from aggregating
+#'   transcript spots should be sparse. When the bins are large, the matrix will
+#'   not be very sparse so using sparse matrix will not save memory, but when
+#'   the bins are small, sparsity is worth it.
+#' @param BPPARAM bpparam object to specify parallel computing over genes. If a
+#'   lot of memory is used, then stick to `SerialParam()`. If \code{save_memory
+#'   = TRUE}, then this argument is ignored because the operation is not thread
+#'   safe; \code{SerialParam()} will always be used.
+#' @param save_memory Logical, if TRUE, then the transcript spots will not all
+#'   be loaded into memory. \code{\link[arrow]{open_dataset}} is used to open a
+#'   link to the data and then transcript spots of one gene is loaded into
+#'   memory at a time.
+#' @param progressbar Logical, whether to show progress bar. This argument is
+#'   only used when \code{save_memory = TRUE} because otherwise the
+#'   \code{progressbar} argument can be specified in \code{BPPARAM}.
+#' @param .orig_nrows Only used internally in the SFE method of \code{aggregate}
 #' @inheritParams formatTxTech
 #' @inheritParams formatTxSpots
 #' @inheritParams readXenium
@@ -34,6 +50,8 @@
 #'   each geometry. Geometries with no spot are removed.
 #' @importFrom data.table as.data.table
 #' @importFrom sf st_make_grid
+#' @importFrom rlang %||%
+#' @importFrom utils tail
 #' @concept Geometric operations
 #' @export
 aggregateTx <- function(file, df = NULL, by = NULL, sample_id = "sample01",
@@ -41,31 +59,122 @@ aggregateTx <- function(file, df = NULL, by = NULL, sample_id = "sample01",
                         gene_col = "gene",
                         phred_col = "qv", min_phred = 20, flip_geometry = FALSE,
                         cellsize = NULL, square = TRUE, flat_topped = FALSE,
-                        new_geometry_name = "bins", unit = "micron") {
+                        new_geometry_name = "bins", unit = "micron", sparse = FALSE,
+                        BPPARAM = SerialParam(), save_memory = FALSE,
+                        progressbar = FALSE,
+                        .orig_nrows = NULL) {
     # This is only for one file, one sample
-    if (!is.null(df)) file <- df
+    if (!is.null(df)) {
+        file <- df
+        save_memory <- FALSE
+    }
     mols <- .check_tx_file(file, spatialCoordsNames, gene_col, phred_col,
-                           min_phred, flip_geometry)
-    mols <- df2sf(mols, spatialCoordsNames = spatialCoordsNames,
-                  geometryType = "POINT")
-    if (is.null(by))
-        by <- st_make_grid(mols, cellsize = cellsize, square = square,
-                           flat_topped = flat_topped)
-    else if (is(by, "sf")) by <- st_geometry(by)
-    grid_sf <- st_sf(grid_id = seq_along(by), geometry = by)
-    mols <- st_join(mols, grid_sf) # Took 5.87 minutes for 7171453 spots and 8555 bins
-    mols <- st_drop_geometry(mols) |> as.data.table()
-    mols <- mols[, .N, by = .(gene, grid_id)]
-    mols$gene <- factor(mols$gene) # The levels are alphabetically arranged
-    mols$gene_index <- as.integer(mols$gene)
-    new_mat <- sparseMatrix(i = mols$gene_index, j = mols$grid_id, x = mols$N)
-    rownames(new_mat) <- levels(mols$gene)
-    colnames(new_mat) <- seq_len(ncol(new_mat))
+                           min_phred, flip_geometry, save_memory = save_memory)
+    if (!save_memory) {
+        if (inherits(mols, "data.table"))
+            mols <- mols[,c(spatialCoordsNames, "gene"), with=FALSE]
+        else mols <- mols[,c(spatialCoordsNames, "gene")]
+        mols <- df2sf(mols, spatialCoordsNames = spatialCoordsNames,
+                      geometryType = "POINT")
+    }
+    if (is.null(by)) {
+        if (save_memory) {
+            mols_bbox <- mols |> 
+                dplyr::summarize(xmin = min(!!sym(spatialCoordsNames[1])),
+                                 xmax = max(!!sym(spatialCoordsNames[1])),
+                                 ymin = min(!!sym(spatialCoordsNames[2])),
+                                 ymax = max(!!sym(spatialCoordsNames[2]))) |> 
+                dplyr::collect() |> unlist() |> st_bbox()
+            by <- st_make_grid(mols_bbox, cellsize = cellsize, square = square,
+                               flat_topped = flat_topped)
+        } else {
+            by <- st_make_grid(mols, cellsize = cellsize, square = square,
+                               flat_topped = flat_topped)
+        }
+    } else if (inherits(by, "sf")) by <- st_geometry(by)
+    if (save_memory) {
+        # TODO: Add argument to write this stuff to disk on the fly gene by gene
+        # never having the whole matrix in memory
+        # Not urgent, since usually the matrix isn't that big anyway after aggregation
+        # TODO: Add argument to set max size loaded into memory. Then instead of
+        # one gene at a time, we can do groups of genes and parallelize within the group
+        # which would really help for less expressed genes. Each point takes a bout 20 bytes or so.
+        # This way it can run faster since each filter operation takes a while
+        # for csv.gz.
+        genes_use <- mols |> 
+            dplyr::select(gene) |> dplyr::distinct() |> 
+            dplyr::pull(gene, as_vector = TRUE)
+        ml <- bplapply(seq_along(genes_use), function(i) {
+            x <- mols |> 
+                dplyr::filter(gene == genes_use[[i]]) |> 
+                dplyr::select(gene, dplyr::any_of(spatialCoordsNames)) |> 
+                dplyr::collect()
+            x <- df2sf(x, spatialCoordsNames = spatialCoordsNames,
+                       geometryType = "POINT")
+            if (sparse) {
+                ll <- lengths(st_intersects(by, x))
+                j <- which(ll > 0) # Need to deal with special case of all 0's when by doesn't cover the whole area
+                data.frame(i = i, j = j, x = ll[j])
+            } else {
+                inds <- st_intersects(by, x)
+                lengths(inds)
+            }
+        }, BPPARAM = SerialParam(progressbar = progressbar))
+        if (sparse) {
+            ml <- data.table::rbindlist(ml)
+            new_mat <- sparseMatrix(i = ml$i, j = ml$j, x = ml$x, dims = c(length(genes_use), length(by)),
+                                    dimnames = list(genes_use, seq_along(by)))
+        } else {
+            new_mat <- matrix(unlist(ml), nrow = length(by), ncol = length(genes_use),
+                              dimnames = list(seq_along(by), genes_use))
+            new_mat <- t(new_mat)
+        }
+    } else {
+        mols <- split(st_geometry(mols), mols[["gene"]])
+        if (sparse) {
+            # Special case from .aggregate_SFE, where numeric L1 is used for genes
+            if (gene_col == "L1") {
+                ml <- bplapply(names(mols), function(i) {
+                    x <- mols[[i]]
+                    ll <- lengths(st_intersects(by, x))
+                    j <- which(ll > 0) # When the spots fall outside all bins
+                    if (!length(j)) return(NULL)
+                    data.frame(i = as.integer(i), j = j, x = ll[j])
+                }, BPPARAM = BPPARAM)
+            } else {
+                ml <- bplapply(seq_along(mols), function(i) {
+                    x <- mols[[i]]
+                    ll <- lengths(st_intersects(by, x))
+                    j <- which(ll > 0)
+                    data.frame(i = i, j = j, x = ll[j])
+                }, BPPARAM = BPPARAM)
+            }
+            ml <- data.table::rbindlist(ml)
+            if (gene_col == "L1") {
+                nrows_use <- .orig_nrows %||% as.integer(tail(names(mols), 1))
+                new_mat <- sparseMatrix(i = ml$i, j = ml$j, x = ml$x, dims = c(nrows_use, length(by)),
+                                        dimnames = list(as.character(seq_len(nrows_use)), seq_along(by)))
+            } else {
+                new_mat <- sparseMatrix(i = ml$i, j = ml$j, x = ml$x, dims = c(length(mols), length(by)),
+                                        dimnames = list(names(mols), seq_along(by)))
+            }
+        } else {
+            ml <- bplapply(mols, function(x) {
+                inds <- st_intersects(by, x)
+                lengths(inds)
+            }, BPPARAM = BPPARAM)
+            new_mat <- matrix(unlist(ml), nrow = length(by), ncol = length(mols),
+                              dimnames = list(seq_along(by), names(mols)))
+            new_mat <- t(new_mat)
+        }
+    }
+    
     new_mat <- new_mat[,colSums(new_mat) > 0] # Remove empty grid cells
-    cgs <- list(bins = grid_sf[grid_sf$grid_id %in% mols$grid_id, "geometry"])
+    grid_sf <- st_sf(geometry = by)
+    cgs <- list(bins = grid_sf[colnames(new_mat), "geometry"])
     names(cgs) <- new_geometry_name
     SpatialFeatureExperiment(assays = list(counts = new_mat),
-                             colGeometries = cgs, unit = unit)
+                             colGeometries = cgs, unit = unit, sample_id = sample_id)
 }
 
 #' @rdname aggregateTx
@@ -77,7 +186,9 @@ aggregateTxTech <- function(data_dir, df = NULL, by = NULL,
                             min_phred = 20, flip = c("geometry", "image", "none"),
                             max_flip = "50 MB",
                             cellsize = NULL, square = TRUE, flat_topped = FALSE,
-                            new_geometry_name = "bins") {
+                            new_geometry_name = "bins", sparse = FALSE,
+                            BPPARAM = SerialParam(), 
+                            save_memory = FALSE, progressbar = FALSE) {
     tech <- match.arg(tech)
     flip <- match.arg(flip)
     c(spatialCoordsNames, gene_col, cell_col, fn) %<-%
@@ -113,7 +224,8 @@ aggregateTxTech <- function(data_dir, df = NULL, by = NULL,
                 phred_col = "qv", min_phred = min_phred,
                 flip_geometry = (flip == "geometry"),
                 cellsize = cellsize, square = square, flat_topped = flat_topped,
-                new_geometry_name = new_geometry_name)
+                new_geometry_name = new_geometry_name, BPPARAM = BPPARAM, 
+                sparse = sparse, save_memory = save_memory, progressbar = progressbar)
     imgData(sfe) <- img_df
     sfe
 }
@@ -144,7 +256,7 @@ aggregateTxTech <- function(data_dir, df = NULL, by = NULL,
         out_agg <- matrix(unlist(out_agg), ncol = length(inds))
         rownames(out_agg) <- rownames(mat)
     } else stop("Function ", fun_name, " not supported for aggregating SFE.")
-    if (is(out_agg, "dgeMatrix")) out_agg <- as.matrix(out_agg)
+    if (inherits(out_agg, "dgeMatrix")) out_agg <- as.matrix(out_agg)
     out_agg
 }
 
@@ -189,6 +301,10 @@ aggregateTxTech <- function(data_dir, df = NULL, by = NULL,
         cat_agg <- matrix(NA, nrow = length(inds), ncol = length(names_not_num))
         colnames(cat_agg) <- names_not_num
         cat_agg <- data.frame(cat_agg)
+        if (nrow(df_bin) != nrow(df)) {
+            df_inds <- data.frame(index = seq_len(nrow(df)))
+            df_bin <- merge(df_inds, df_bin, all.x = TRUE, by = "index")
+        }
         for (n in names_not_num)
             cat_agg[[n]] <- split(df[[n]], df_bin$bin)
         cd_agg <- cbind(cat_agg, cd_agg)
@@ -204,30 +320,23 @@ aggregateTxTech <- function(data_dir, df = NULL, by = NULL,
 }
 
 # Might turn this into an exported function
-.aggregate_sample_tx <- function(x, by, rowGeometryName, new_geometry_name) {
+.aggregate_sample_tx <- function(x, by, rowGeometryName, new_geometry_name, 
+                                 sparse = FALSE, BPPARAM = SerialParam(),
+                                 .orig_nrows = NULL) {
     rg <- rowGeometry(x, rowGeometryName)
     if (!is.null(st_z_range(rg)))
-        by <- st_zm(by, drop = FALSE, what = "Z")
+        rg <- st_zm(rg)
+    if (st_geometry_type(rg, by_geometry = FALSE) == "GEOMETRY") {
+        # Happens after cropping and producing empty geometries
+        rg <- st_cast(rg, "MULTIPOINT") |> st_zm()
+    }
     grid_sf <- st_sf(grid_id = seq_along(by), geometry = by)
-
-    # Probably faster than directly calling st_intersection, since I don't need
-    # the actual geometries of the intersections, maybe not
-    tx_coords <- st_coordinates(rg) # Might write another function similar to formatTxTech to skip this
-    tx_point <- df2sf(tx_coords, spatialCoordsNames = c("X", "Y", "Z"))
-    tx_ind <- as.data.table(st_drop_geometry(tx_point))
-    tx_info <- txSpots(x) |> st_drop_geometry()
-    tx_info$L1 <- seq_along(tx_info$gene) # it has to be "gene" if it's from formatTxSpots
-    tx_point <- merge(tx_point, tx_info, by = "L1") # takes a while
-    tx_point <- st_as_sf(tx_point) |> st_join(grid_sf) # takes a few minutes
-    tx_counts <- tx_point |> st_drop_geometry() |> as.data.table()
-    tx_counts <- tx_counts[, .N, by = .(gene, L1, grid_id)]
-
-    new_mat <- sparseMatrix(i = tx_counts$L1, j = tx_counts$grid_id, x = tx_counts$N)
-    cgs <- list(bins = grid_sf)
-    names(cgs) <- new_geometry_name
-    out <- SpatialFeatureExperiment(assays = list(counts = new_mat),
-                             colGeometries = cgs)
-    colnames(out) <- seq_along(by)
+    tx_coords <- st_coordinates(rg) |> as.data.frame()
+    scn <- c("X", "Y")
+    out <- aggregateTx(df = tx_coords, spatialCoordsNames = scn,
+                       gene_col = "L1", by = by, sparse = sparse,
+                       BPPARAM = BPPARAM, .orig_nrows = .orig_nrows,
+                       sample_id = sampleIDs(x))
     rownames(out) <- rownames(x)
     out
 }
@@ -235,66 +344,78 @@ aggregateTxTech <- function(data_dir, df = NULL, by = NULL,
 .aggregate_sample <- function(x, by = NULL, FUN = sum, fun_name,
                               colGeometryName = 1L, rowGeometryName = NULL,
                               join = st_intersects, new_geometry_name = "bins",
-                              BPPARAM = SerialParam()) {
+                              sparse = FALSE,
+                              BPPARAM = SerialParam(), .orig_nrows = NULL) {
     # Here x is an SFE object with one sample
     # by is sfc
     # Can't do S4 method with signature for `by` because the argument `by` isn't
     # in the generic and I don't want to mess with the `aggregate` function in
     # other packages
     if (!is.null(rowGeometryName)) {
-        .aggregate_sample_tx(x, by, rowGeometryName, new_geometry_name)
+        .aggregate_sample_tx(x, by, rowGeometryName, new_geometry_name, 
+                             sparse = sparse, BPPARAM = BPPARAM, 
+                             .orig_nrows = .orig_nrows)
     } else {
         .aggregate_sample_cell(x, by, FUN, fun_name, colGeometryName, join,
                                new_geometry_name, BPPARAM)
     }
 }
 
-.aggregate_SFE <-
-    function(x, by = NULL, FUN = sum, sample_id = "all",
-             colGeometryName = 1L, rowGeometryName = NULL,
-             cellsize = NULL, square = TRUE, flat_topped = FALSE,
-             new_geometry_name = "bins", join = st_intersects,
-             BPPARAM = SerialParam()) {
-        sample_id <- .check_sample_id(x, sample_id, one = FALSE)
-        if (is.null(by) && is.null(cellsize)) {
-            stop("Either `by` or `cellsize` must be specified.")
-        }
-        # Make grid for multiple samples if `by` is not specified
-        if (is.null(by)) {
-            by <- .make_grid_samples(x, sample_id,
-                                     cellsize, square, flat_topped)
-        }
-        if (is.list(by) && !is(by, "sfc")) {
-            if (!any(sample_id %in% names(by)))
-                stop("None of the geometries in `by` correspond to sample_id")
-            by <- by[intersect(sample_id, names(by))]
-        } else {
-            if (!is(by, "sfc") && !is(by, "sf"))
-                stop("`by` must be either sf or sfc.")
-            if (length(sample_id) > 1L) {
-                if (is(by, "sfc") || !"sample_id" %in% names(by))
-                    stop("`by` must be an sf data frame with a column `sample_id`")
-                by <- split(st_geometry(by), by$sample_id)
-            }
-        }
-        if (is(by, "sfc")) by <- setNames(list(by), sample_id)
-        fun_name <- as.character(substitute(FUN))
-        sfes <- splitSamples(x) # Output list should have sample IDs as names
-        sfes <- lapply(sample_id, function(s) {
-            .aggregate_sample(sfes[[s]], by = by[[s]], FUN = FUN,
-                              colGeometryName = colGeometryName,
-                              rowGeometryName = rowGeometryName,
-                              join = join, fun_name = fun_name,
-                              new_geometry_name = new_geometry_name)
-        })
-        out <- do.call(cbind, sfes)
-        # Add the original rowGeometries back
-        rowGeometries(out) <- rowGeometries(x, sample_id = sample_id)
-        # Keep imgData
-        id_orig <- imgData(x)
-        imgData(out) <- id_orig[id_orig$sample_id %in% sample_id,]
-        out
+.aggregate_SFE <- function(x, by = NULL, FUN = sum, sample_id = "all",
+                           colGeometryName = 1L, rowGeometryName = NULL,
+                           cellsize = NULL, square = TRUE, flat_topped = FALSE,
+                           new_geometry_name = "bins", join = st_intersects,
+                           sparse = FALSE,
+                           BPPARAM = SerialParam()) {
+    sample_id <- .check_sample_id(x, sample_id, one = FALSE)
+    if (is.null(by) && is.null(cellsize)) {
+        stop("Either `by` or `cellsize` must be specified.")
     }
+    # Make grid for multiple samples if `by` is not specified
+    if (is.null(by)) {
+        by <- .make_grid_samples(x, sample_id,
+                                 cellsize, square, flat_topped)
+    }
+    if (is.list(by) && !inherits(by, "sfc") && !inherits(by, "sf")) {
+        if (!any(sample_id %in% names(by)))
+            stop("None of the geometries in `by` correspond to sample_id")
+        by <- by[intersect(sample_id, names(by))]
+    } else {
+        if (!inherits(by, "sfc") && !inherits(by, "sf"))
+            stop("`by` must be either sf or sfc.")
+        if (length(sample_id) > 1L) {
+            if (inherits(by, "sfc") || !"sample_id" %in% names(by))
+                stop("`by` must be an sf data frame with a column `sample_id`")
+            by <- split(st_geometry(by), by$sample_id)
+        } else if (inherits(by, "sf")) {
+            by <- st_geometry(by)
+        }
+    }
+    if (inherits(by, "sfc")) by <- setNames(list(by), sample_id)
+    fun_name <- as.character(substitute(FUN))
+    sfes <- splitSamples(x) # Output list should have sample IDs as names
+    if (!is.null(rowGeometryName)) {
+        any_empty <- vapply(sfes, function(x) any(st_is_empty(rowGeometry(x, rowGeometryName))),
+                            FUN.VALUE = logical(1)) |> any()
+        if (any_empty) sparse <- TRUE
+    }
+    sfes <- lapply(sample_id, function(s) {
+        .aggregate_sample(sfes[[s]], by = by[[s]], FUN = FUN,
+                          colGeometryName = colGeometryName,
+                          rowGeometryName = rowGeometryName,
+                          join = join, fun_name = fun_name,
+                          new_geometry_name = new_geometry_name,
+                          sparse = sparse, BPPARAM = BPPARAM, 
+                          .orig_nrows = nrow(sfes[[s]]))
+    })
+    out <- do.call(cbind, sfes)
+    # Add the original rowGeometries back
+    rowGeometries(out) <- rowGeometries(x, sample_id = sample_id)
+    # Keep imgData
+    id_orig <- imgData(x)
+    imgData(out) <- id_orig[id_orig$sample_id %in% sample_id,]
+    out
+}
 
 #' Aggregate data in SFE using geometry
 #'
@@ -320,6 +441,7 @@ aggregateTxTech <- function(data_dir, df = NULL, by = NULL,
 #'
 #' @inheritParams sf::st_make_grid
 #' @inheritParams sf::aggregate
+#' @inheritParams aggregateTx
 #' @param x An SFE object to be aggregated.
 #' @param by A \code{sf} data frame whose geometry column is used for
 #'   aggregation or \code{sfc} or for multiple samples a list of \code{sfc}
@@ -343,9 +465,10 @@ aggregateTxTech <- function(data_dir, df = NULL, by = NULL,
 #' @param rowGeometryName Which \code{rowGeometry} to spatially aggregate
 #' @param new_geometry_name Name to give to the new \code{colGeometry} in the
 #'   output. Defaults to "bins".
-#' @param BPPARAM A \code{\link{BiocParallelParam}} object specifying parallel
-#'   computing when aggregating data with functions other than sum and mean.
-#'   Defaults to \code{SerialParam()}.
+#' @param BPPARAM A \code{\link[BiocParallel]{BiocParallelParam}} object specifying parallel
+#'   computing when aggregating data with functions other than sum and mean when
+#'   aggregating cells. When aggregating transcript spots, this specifies
+#'   parallel computing over genes. Defaults to \code{SerialParam()}.
 #' @return An SFE object with \code{colGeometry} the same as the geometry
 #'   specified in \code{by} or same as the grid specified in \code{cellsize}.
 #'   \code{rowGeometries} and \code{rowData} remain the same as in the input
@@ -365,7 +488,7 @@ aggregateTxTech <- function(data_dir, df = NULL, by = NULL,
 #' @concept Geometric operations
 #' @examples
 #' # example code
-#'
+#' 
 setMethod("aggregate", "SpatialFeatureExperiment", .aggregate_SFE)
 
 # Function to make grid for multiple samples
