@@ -14,7 +14,10 @@
 #' Spatial neighborhood graphs as \code{spdep}'s \code{listw} objects are stored
 #' in the \code{int_metadata} of the SFE object. The \code{listw} class is used
 #' because \code{spdep} has many useful methods that rely on the neighborhood
-#' graph as \code{listw}.
+#' graph as \code{listw}. See the
+#' \href{https://r-spatial.github.io/spdep/reference/index.html}{\code{spdep}
+#' doumentation website} for functions to edit the spatial neighborhood graph,
+#' or the \code{nb} object within the \code{listw}.
 #'
 #' @param x A \code{SpatialFeatureExperiment} object.
 #' @param value A \code{listw} object (\code{*Graph}), or a named list of list
@@ -35,12 +38,12 @@
 #'   useful when multiple pieces of tissues are in the same SFE object (say for
 #'   a joint dimension reduction and clustering) and the spatial neighborhood is
 #'   only meaningful within the same piece of tissue. See the \code{sample_id}
-#'   argument in \code{\link{SpatialExperiment}}.
+#'   argument in \code{\link[SpatialExperiment]{SpatialExperiment}}.
 #' @name spatialGraphs
 #' @concept Getters and setters
-#' @return Getters for multiple graphs return a named list. Getters for
-#'   names return a character vector of the names. Getters for single graphs
-#'   return a \code{listw} object. Setters return an SFE object.
+#' @return Getters for multiple graphs return a named list. Getters for names
+#'   return a character vector of the names. Getters for single graphs return a
+#'   \code{listw} object. Setters return an SFE object.
 #' @aliases rowGraphs rowGraphs<- spatialGraph spatialGraph<- spatialGraphNames
 #'   colGraphs colGraphs<- spatialGraphNames<- spatialGraphs<- annotGraphs
 #'   annotGraphs<-
@@ -231,11 +234,11 @@ annotGraphs <- function(x, sample_id = "all", name = "all")
     if (type == "all") {
         if (is.null(value)) {
             df <- .initialize_spatialGraphs(x)
-        } else if (!is(value, "DataFrame")) {
+        } else if (!inherits(value, "DataFrame")) {
             value <- lapply(value, .fill_missing,
                 names_use = c("row", "col", "annot")
             )
-            df <- DataFrame(lapply(value, I), row.names = c("row", "col", "annot"))
+            df <- DataFrame(lapply(value, I), row.names = c("row", "col", "annot"), check.names = FALSE)
         } else {
             df <- value[c("row", "col", "annot"), , drop = FALSE]
         }
@@ -425,7 +428,7 @@ annotGraph <- function(x, type = 1L, sample_id = 1L) {
 .sg_r <- function(x, type = 1L, MARGIN, sample_id = NULL, value) {
     sample_id <- .check_sample_id(x, sample_id)
     if (!is.null(value)) {
-        if (!is(value, "listw")) {
+        if (!inherits(value, "listw")) {
             stop("value must be of class listw.")
         } else if (MARGIN == 1L && length(value$neighbours) != nrow(x)) {
             stop(
