@@ -7,8 +7,16 @@ pts = st_sfc(st_point(c(.5,.5)), st_point(c(1.5, 1.5)), st_point(c(2.5, 2.5)))
 pol = st_polygon(list(rbind(c(0,0), c(2,0), c(2,2), c(0,2), c(0,0))))
 
 test_that("trivial, st_any_intersects", {
-    o <- st_any_intersects(pts, pol)
+    o <- st_any_intersects(pts, pol, sparse = FALSE)
     expect_equal(o, c(TRUE, TRUE, FALSE))
+    o <- st_any_intersects(pts, pol, sparse = TRUE)
+    expect_equal(o, c(1L, 2L))
+})
+
+test_that("Swap x and y for st_intersects", {
+    o1 <- st_any_intersects(pts, pol, sparse = TRUE)
+    o2 <- st_any_intersects(pts, pol, sparse = TRUE, yx = TRUE)
+    expect_equal(o1, o2)
 })
 
 test_that("trivial, st_n_intersects", {
@@ -48,10 +56,10 @@ rowGeometry(sfe_visium, "points", sample_id = "sample01", withDimnames = FALSE)
 test_that("All spots in the cropped SFE objects indeed are covered by the bbox", {
     sfe_cropped <- SpatialFeatureExperiment::crop(sfe_visium, bbox_use, sample_id = "all")
     cg <- spotPoly(sfe_cropped, "all")
-    expect_true(all(st_any_pred(cg, bbox_use, pred = st_covered_by)))
+    expect_true(all(st_any_pred(cg, bbox_use, pred = st_covered_by, sparse = FALSE)))
     expect_true(st_geometry_type(cg, by_geometry = FALSE) == "POLYGON")
     rg <- rowGeometry(sfe_cropped)
-    expect_true(all(st_any_pred(rg, bbox_use, pred = st_covered_by)))
+    expect_true(all(st_any_pred(rg, bbox_use, pred = st_covered_by, sparse = FALSE)))
 })
 
 test_that("Only crop one sample out of two, with sf", {
@@ -59,10 +67,10 @@ test_that("Only crop one sample out of two, with sf", {
                  "No bounding boxes for samples specified.")
     sfe_cropped2 <- SpatialFeatureExperiment::crop(sfe_visium, y = bbox_sf, sample_id = "sample01")
     expect_true(all(st_any_pred(spotPoly(sfe_cropped2, "sample01"), bbox_use,
-                                pred = st_covered_by
+                                pred = st_covered_by, sparse = FALSE
     )))
     expect_false(all(st_any_pred(spotPoly(sfe_cropped2, "sample02"), bbox_use,
-                                 pred = st_covered_by
+                                 pred = st_covered_by, sparse = FALSE
     )))
     expect_equal(sum(st_any_intersects(
         spotPoly(sfe_cropped2, "sample02"),
@@ -92,10 +100,10 @@ test_that("Using a bounding box to crop SFE objects, current way, one sample", {
                  "No bounding boxes for samples specified.")
     sfe_cropped <- SpatialFeatureExperiment::crop(sfe_visium, y = m)
     expect_true(all(st_any_pred(spotPoly(sfe_cropped, "sample01"), bbox_use,
-                                pred = st_covered_by
+                                pred = st_covered_by, sparse = FALSE
     )))
     expect_false(all(st_any_pred(spotPoly(sfe_cropped, "sample02"), bbox_use,
-                                 pred = st_covered_by
+                                 pred = st_covered_by, sparse = FALSE
     )))
     expect_equal(sum(st_any_intersects(
         spotPoly(sfe_cropped, "sample02"),
@@ -111,10 +119,10 @@ test_that("Using a bounding box to crop SFE objects, current way, all samples",
                                 c("sample01", "sample02")))
     sfe_cropped <- SpatialFeatureExperiment::crop(sfe_visium, y = m)
     expect_true(all(st_any_pred(spotPoly(sfe_cropped, "sample01"), bbox_use,
-                                pred = st_covered_by
+                                pred = st_covered_by, sparse = FALSE
     )))
     expect_true(all(st_any_pred(spotPoly(sfe_cropped, "sample02"), bbox_use2,
-                                pred = st_covered_by
+                                pred = st_covered_by, sparse = FALSE
     )))
 })
 
@@ -143,10 +151,10 @@ test_that("Keep whole colGeometry items", {
     cg2 <- colGeometry(sfe_cropped, sample_id = "sample01")
     # colGeometry not cropped
     expect_equal(cg, cg2)
-    # annotGeometry cropped by bbox of colGeometry
+    # annotGeometry cropped
     cg_bbox <- st_bbox(cg) |> st_as_sfc()
     ag2 <- annotGeometry(sfe_cropped, sample_id = "sample01")
-    expect_true(st_equals(ag2, cg_bbox, sparse = FALSE))
+    expect_true(st_equals(ag2, st_as_sfc(bbox_cg), sparse = FALSE))
     # rowGeometry
     rg_check <- rowGeometry(sfe_cropped, "points", sample_id = "sample01")
     rg_bbox2 <- st_bbox(rg_check) |> st_as_sfc()
@@ -289,7 +297,7 @@ test_that("annotOp", {
         rownames(out),
         colnames(sfe_visium)[colData(sfe_visium)$sample_id == "sample01"]
     )
-    p <- st_any_pred(out, bbox_use, st_covered_by)
+    p <- st_any_pred(out, bbox_use, st_covered_by, sparse = FALSE)
     expect_true(all(p[c(1, 2, 5)]))
     expect_false(any(p[3:4]))
 })
@@ -484,23 +492,23 @@ test_that("annotSummary", {
 
 # Operations when there're images=================
 # Need uncropped image
-if (!dir.exists("ob")) dir.create(file.path("ob", "outs"), recursive = TRUE)
-mat_fn <- file.path("ob", "outs", "filtered_feature_bc_matrix.h5")
+if (!dir.exists("ob")) dir.create("ob", recursive = TRUE)
+mat_fn <- file.path("ob", "filtered_feature_bc_matrix.h5")
 if (!file.exists(mat_fn))
     download.file("https://cf.10xgenomics.com/samples/spatial-exp/2.0.0/Visium_Mouse_Olfactory_Bulb/Visium_Mouse_Olfactory_Bulb_filtered_feature_bc_matrix.h5",
-                  destfile = file.path("ob", "outs", "filtered_feature_bc_matrix.h5"),
+                  destfile = file.path("ob", "filtered_feature_bc_matrix.h5"),
                   mode = "wb")
-if (!dir.exists(file.path("ob", "outs", "spatial"))) {
+if (!dir.exists(file.path("ob", "spatial"))) {
     download.file("https://cf.10xgenomics.com/samples/spatial-exp/2.0.0/Visium_Mouse_Olfactory_Bulb/Visium_Mouse_Olfactory_Bulb_spatial.tar.gz",
-                  destfile = file.path("ob", "outs", "spatial.tar.gz"))
-    untar(file.path("ob", "outs", "spatial.tar.gz"), exdir = file.path("ob", "outs"))
+                  destfile = file.path("ob", "spatial.tar.gz"))
+    untar(file.path("ob", "spatial.tar.gz"), exdir = "ob")
 }
 
 library(SpatialExperiment)
 library(terra)
 library(SingleCellExperiment)
 library(S4Vectors)
-sfe <- read10xVisiumSFE("ob")
+sfe <- read10xVisiumSFE(dirs = "ob")
 
 test_that("bbox when images are included", {
     bbox_tot <- bbox(sfe, include_image = TRUE) |> st_bbox() |> st_as_sfc()
@@ -536,12 +544,12 @@ test_that("When no cells/spots left after cropping", {
 test_that("Image is shifted after removing empty space", {
     sfe2 <- removeEmptySpace(sfe)
     cg <- df2sf(spatialCoords(sfe2), spatialCoordsNames(sfe2))
-    img <- imgRaster(getImg(sfe2))
+    img <- getImg(sfe2)
     v <- terra::extract(terra::mean(img), cg)
     nCounts <- Matrix::colSums(counts(sfe2))
     expect_true(abs(cor(nCounts, v$mean)) > 0.4)
     bbox_geom <- st_bbox(spotPoly(sfe2)) |> st_as_sfc()
-    bbox_img <- as.vector(ext(img)) |> st_bbox() |> st_as_sfc()
+    bbox_img <- ext(img) |> st_bbox() |> st_as_sfc()
     expect_true(st_covered_by(bbox_geom, bbox_img, sparse = FALSE))
     expect_true(st_area(bbox_geom) / st_area(bbox_img) > 0.97)
 })
@@ -554,12 +562,12 @@ test_that("Image is cropped after cropping SFE object", {
         st_bbox() |> st_as_sfc()
     sfe2 <- SpatialFeatureExperiment::crop(sfe, bbox_use)
     cg <- df2sf(spatialCoords(sfe2), spatialCoordsNames(sfe2))
-    img <- imgRaster(getImg(sfe2, image_id = "hires"))
+    img <- getImg(sfe2, image_id = "hires")
     v <- terra::extract(terra::mean(img), cg)
     nCounts <- Matrix::colSums(counts(sfe2))
     expect_true(abs(cor(nCounts, v$mean, use = "complete.obs")) > 0.4)
     bbox_geom <- st_bbox(spotPoly(sfe2)) |> st_as_sfc()
-    bbox_img <- as.vector(ext(img)) |> st_bbox() |> st_as_sfc()
+    bbox_img <- ext(img) |> st_bbox() |> st_as_sfc()
     expect_true(st_covered_by(bbox_geom, bbox_img, sparse = FALSE))
     expect_true(st_area(bbox_geom) / st_area(bbox_img) > 0.99)
 })
@@ -568,7 +576,7 @@ test_that("Image is cropped after cropping SFE object", {
 test_that("Transpose SFE object with image", {
     sfe2 <- transpose(sfe)
     cg <- df2sf(spatialCoords(sfe2), spatialCoordsNames(sfe2))
-    img <- imgRaster(getImg(sfe2))
+    img <- getImg(sfe2)
     v <- terra::extract(terra::mean(img), cg)
     nCounts <- Matrix::colSums(counts(sfe2))
     expect_true(abs(cor(nCounts, v$mean)) > 0.4)
@@ -585,7 +593,7 @@ test_that("Transpose SFE object with image, after cropping image", {
     sfe <- sfe[,sfe$in_tissue]
     sfe2 <- transpose(sfe)
     cg <- df2sf(spatialCoords(sfe2), spatialCoordsNames(sfe2))
-    img <- imgRaster(getImg(sfe2, image_id = "hires"))
+    img <- getImg(sfe2, image_id = "hires")
     v <- terra::extract(terra::mean(img), cg)
     nCounts <- Matrix::colSums(counts(sfe2))
     expect_true(abs(cor(nCounts, v$mean)) > 0.4)
@@ -596,7 +604,7 @@ test_that("Transpose SFE object with image, after cropping image", {
     bbox_img_orig_sf <- bbox_img_orig |> st_bbox() |> st_as_sfc()
     bbox_cg <- st_bbox(spotPoly(sfe2))
     bbox_cg_sf <- bbox_cg |> st_as_sfc()
-    bbox_img <- as.vector(ext(img))
+    bbox_img <- ext(img)
     bbox_img_sf <- bbox_img |> st_bbox() |> st_as_sfc()
     expect_true(st_covered_by(bbox_cg_sf, bbox_img_sf, sparse = FALSE))
     expect_equal(bbox_img_orig[["ymax"]] - bbox_cg_orig[["ymax"]],
@@ -613,7 +621,7 @@ test_that("Transpose SFE object with image, after cropping image", {
 test_that("Mirror SFE object with image, vertical", {
     sfe2 <- mirror(sfe, direction = "vertical")
     cg <- df2sf(spatialCoords(sfe2), spatialCoordsNames(sfe2))
-    img <- imgRaster(getImg(sfe2))
+    img <- getImg(sfe2)
     v <- terra::extract(terra::mean(img), cg)
     nCounts <- Matrix::colSums(counts(sfe2))
     expect_true(abs(cor(nCounts, v$mean)) > 0.4)
@@ -630,7 +638,7 @@ test_that("Mirror SFE object with image after cropping", {
     sfe <- sfe[,sfe$in_tissue]
     sfe2 <- mirror(sfe, direction = "vertical")
     cg <- df2sf(spatialCoords(sfe2), spatialCoordsNames(sfe2))
-    img <- imgRaster(getImg(sfe2, image_id = "hires"))
+    img <- getImg(sfe2, image_id = "hires")
     v <- terra::extract(terra::mean(img), cg)
     nCounts <- Matrix::colSums(counts(sfe2))
     expect_true(abs(cor(nCounts, v$mean)) > 0.4)
@@ -641,7 +649,7 @@ test_that("Mirror SFE object with image after cropping", {
     bbox_img_orig_sf <- bbox_img_orig |> st_bbox() |> st_as_sfc()
     bbox_cg <- st_bbox(spotPoly(sfe2))
     bbox_cg_sf <- bbox_cg |> st_as_sfc()
-    bbox_img <- as.vector(ext(img))
+    bbox_img <- ext(img)
     bbox_img_sf <- bbox_img |> st_bbox() |> st_as_sfc()
     expect_true(st_covered_by(bbox_cg_sf, bbox_img_sf, sparse = FALSE))
     expect_equal(bbox_img_orig[["ymax"]] - bbox_cg_orig[["ymax"]],
@@ -658,7 +666,7 @@ test_that("Mirror SFE object with image after cropping", {
 test_that("Mirror SFE object with image, horizontal", {
     sfe2 <- mirror(sfe, direction = "horizontal")
     cg <- df2sf(spatialCoords(sfe2), spatialCoordsNames(sfe2))
-    img <- imgRaster(getImg(sfe2))
+    img <- getImg(sfe2)
     v <- terra::extract(terra::mean(img), cg)
     nCounts <- Matrix::colSums(counts(sfe2))
     expect_true(abs(cor(nCounts, v$mean)) > 0.4)
@@ -674,7 +682,7 @@ test_that("Mirror SFE object with image, horizontal", {
 test_that("Rotate SFE object with image", {
     sfe2 <- SpatialFeatureExperiment::rotate(sfe, degrees = 45)
     cg <- df2sf(spatialCoords(sfe2), spatialCoordsNames(sfe2))
-    img <- getImg(sfe2) |> toSpatRasterImage(save_geotiff = FALSE) |> imgRaster()
+    img <- getImg(sfe2) |> toSpatRasterImage(save_geotiff = FALSE)
 
     v <- terra::extract(terra::mean(img), cg)
     nCounts <- Matrix::colSums(counts(sfe2))
@@ -698,7 +706,7 @@ test_that("Rotate SFE object with image after cropping", {
     sfe <- sfe[,sfe$in_tissue]
     sfe2 <- SpatialFeatureExperiment::rotate(sfe, degrees = 45)
     cg <- df2sf(spatialCoords(sfe2), spatialCoordsNames(sfe2))
-    img <- getImg(sfe2) |> toSpatRasterImage(save_geotiff = FALSE) |> imgRaster()
+    img <- getImg(sfe2) |> toSpatRasterImage(save_geotiff = FALSE)
 
     v <- terra::extract(terra::mean(img), cg)
     nCounts <- Matrix::colSums(counts(sfe2))
@@ -721,7 +729,7 @@ test_that("Rotate SFE object with image after cropping", {
 test_that("Scale SFE object with image", {
     sfe2 <- SpatialFeatureExperiment::scale(sfe, factor = 1.5)
     cg <- df2sf(spatialCoords(sfe2), spatialCoordsNames(sfe2))
-    img <- getImg(sfe2) |> imgRaster()
+    img <- getImg(sfe2)
 
     v <- terra::extract(terra::mean(img), cg)
     nCounts <- Matrix::colSums(counts(sfe2))
@@ -745,7 +753,7 @@ test_that("Scale SFE object with image after cropping", {
     sfe <- sfe[,sfe$in_tissue]
     sfe2 <- SpatialFeatureExperiment::scale(sfe, factor = 1.5)
     cg <- df2sf(spatialCoords(sfe2), spatialCoordsNames(sfe2))
-    img <- getImg(sfe2) |> imgRaster()
+    img <- getImg(sfe2)
 
     v <- terra::extract(terra::mean(img), cg)
     nCounts <- Matrix::colSums(counts(sfe2))
@@ -770,7 +778,7 @@ test_that("General affine transformation of SFE object with image", {
     v <- c(0, 300)
     sfe2 <- SpatialFeatureExperiment::affine(sfe, M = M, v = v)
     cg <- df2sf(spatialCoords(sfe2), spatialCoordsNames(sfe2))
-    img <- getImg(sfe2) |> toSpatRasterImage(save_geotiff = FALSE) |> imgRaster()
+    img <- getImg(sfe2) |> toSpatRasterImage(save_geotiff = FALSE)
 
     v <- terra::extract(terra::mean(img), cg)
     nCounts <- Matrix::colSums(counts(sfe2))
@@ -796,7 +804,7 @@ test_that("Affine transformation of SFE object with image, after cropping", {
     sfe <- sfe[,sfe$in_tissue]
     sfe2 <- SpatialFeatureExperiment::affine(sfe, M = M, v = v)
     cg <- df2sf(spatialCoords(sfe2), spatialCoordsNames(sfe2))
-    img <- getImg(sfe2) |> toSpatRasterImage(save_geotiff = FALSE) |> imgRaster()
+    img <- getImg(sfe2) |> toSpatRasterImage(save_geotiff = FALSE)
 
     v <- terra::extract(terra::mean(img), cg)
     nCounts <- Matrix::colSums(counts(sfe2))
@@ -842,7 +850,7 @@ test_that("Translate SFE object with image", {
     expect_equal(bbox_tr, bbox_exp)
 
     cg <- df2sf(spatialCoords(sfe2), spatialCoordsNames(sfe2))
-    img <- imgRaster(getImg(sfe2))
+    img <- getImg(sfe2)
     v <- terra::extract(terra::mean(img), cg)
     nCounts <- Matrix::colSums(counts(sfe2))
     expect_true(abs(cor(nCounts, v$mean)) > 0.4)
@@ -860,7 +868,6 @@ library(RBioFormats)
 library(EBImage)
 fp <- tempfile()
 xenium_path <- XeniumOutput(file_path = fp)
-try(sfe <- readXenium(xenium_path))
 sfe <- readXenium(xenium_path, add_molecules = TRUE)
 set.seed(29)
 annotGeometry(sfe, "foo") <- ag <-
@@ -898,9 +905,9 @@ test_that("Transpose SFE object with BioFormatsImage", {
 
 test_that("Mirror SFE object with BFI, vertical", {
     sfe2 <- mirror(sfe, direction = "vertical")
-    img <- imgRaster(getImg(sfe2), resolution = 1L)
+    img <- toExtImage(getImg(sfe2), resolution = 1L)
     mask <- img > 500
-    spi <- ExtImage(mask, ext = ext(getImg(sfe2))) |> toSpatRasterImage(save_geotiff = FALSE)
+    spi <- toSpatRasterImage(mask, save_geotiff = FALSE)
     v <- terra::extract(spi, vect(st_centroid(nucSeg(sfe2)$geometry)))
     expect_true(mean(v$lyr.1, na.rm = TRUE) > 0.9)
 
@@ -922,9 +929,9 @@ test_that("Mirror SFE object with BFI, vertical", {
 
 test_that("Mirror SFE object with BFI, horizontal", {
     sfe2 <- mirror(sfe, direction = "horizontal")
-    img <- imgRaster(getImg(sfe2), resolution = 1L)
+    img <- toExtImage(getImg(sfe2), resolution = 1L)
     mask <- img > 500
-    spi <- ExtImage(mask, ext = ext(getImg(sfe2))) |> toSpatRasterImage(save_geotiff = FALSE)
+    spi <- toSpatRasterImage(mask, save_geotiff = FALSE)
     v <- terra::extract(spi, vect(st_centroid(nucSeg(sfe2)$geometry)))
     expect_true(mean(v$lyr.1, na.rm = TRUE) > 0.9)
 
@@ -946,9 +953,9 @@ test_that("Mirror SFE object with BFI, horizontal", {
 
 test_that("Rotate SFE object with BFI", {
     sfe2 <- SpatialFeatureExperiment::rotate(sfe, degrees = 45)
-    img <- imgRaster(getImg(sfe2), resolution = 1L)
+    img <- toExtImage(getImg(sfe2), resolution = 1L)
     mask <- img > 500
-    spi <- ExtImage(mask, ext = ext(getImg(sfe2))) |> toSpatRasterImage(save_geotiff = FALSE)
+    spi <- toSpatRasterImage(mask, save_geotiff = FALSE)
     v <- terra::extract(spi, vect(st_centroid(nucSeg(sfe2)$geometry)))
     expect_true(mean(v$lyr.1, na.rm = TRUE) > 0.9)
 
@@ -963,9 +970,9 @@ test_that("Rotate SFE object with BFI", {
 
 test_that("Scale SFE object with BFI", {
     sfe2 <- SpatialFeatureExperiment::scale(sfe, factor = 1.5)
-    img <- imgRaster(getImg(sfe2), resolution = 1L)
+    img <- toExtImage(getImg(sfe2), resolution = 1L)
     mask <- img > 500
-    spi <- ExtImage(mask, ext = ext(getImg(sfe2))) |> toSpatRasterImage(save_geotiff = FALSE)
+    spi <- toSpatRasterImage(mask, save_geotiff = FALSE)
     v <- terra::extract(spi, vect(st_centroid(nucSeg(sfe2)$geometry)))
     expect_true(mean(v$lyr.1, na.rm = TRUE) > 0.9)
 
@@ -989,9 +996,9 @@ test_that("General affine transformation of SFE object with BFI", {
     M <- matrix(c(0.6, -0.2, 0.2, 0.3), nrow = 2)
     v <- c(0, 300)
     sfe2 <- SpatialFeatureExperiment::affine(sfe, M = M, v = v)
-    img <- imgRaster(getImg(sfe2), resolution = 1L)
+    img <- toExtImage(getImg(sfe2), resolution = 1L)
     mask <- img > 500
-    spi <- ExtImage(mask, ext = ext(getImg(sfe2))) |> toSpatRasterImage(save_geotiff = FALSE)
+    spi <- toSpatRasterImage(mask, save_geotiff = FALSE)
     v <- terra::extract(spi, vect(st_centroid(nucSeg(sfe2)$geometry)))
     expect_true(mean(v$lyr.1, na.rm = TRUE) > 0.9)
 
